#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <stdio.h>

static Bool set_urgent_hint(Display* dpy, Window wnd, Bool set)
{
	XWMHints *hints = XAllocWMHints();

	if (!hints) {
		fputs("error allocing WM hints!\n", stderr);
		return False;
	}

	printf("%s urgent hint\n", set ? "setting" : "clearing");

	if (set) {
		hints->flags |= XUrgencyHint;
	} else {
		hints->flags &= ~XUrgencyHint;
	}

	XSetWMHints(dpy, wnd, hints);
	XFree(hints);
	return True;
}

static Display* dpy;
static Window wnd;
static Bool urgent = False;

static void handle_sigint(int signo)
{
	if (signo != SIGINT)
		return;

	if (set_urgent_hint(dpy, wnd, !urgent)) {
		urgent = !urgent;
		XStoreName(dpy, wnd, urgent ? "URGENT!" : "bored...");
		XFlush(dpy);
	}
}

int main(void)
{
	int run = 1, status = EXIT_FAILURE;
	Atom atom_wm_delete;
	struct sigaction sa;
	XEvent e;

	dpy = XOpenDisplay(NULL);
	if (!dpy) {
		fputs("cannot open display!\n", stderr);
		return EXIT_FAILURE;
	}

	atom_wm_delete = XInternAtom(dpy, "WM_DELETE_WINDOW", True);
	if (atom_wm_delete == None) {
		fputs("cannot find WM_DELETE_WINDOW atom\n", stderr);
		return EXIT_FAILURE;
	}

	/* create window */
	wnd = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy),
				  0, 0, 200, 100, 0, 0, 0);
	if (!wnd) {
		fputs("cannot create window!\n", stderr);
		goto outdpy;
	}

	XSelectInput(dpy, wnd, ExposureMask | StructureNotifyMask);
	XSetWMProtocols(dpy, wnd, &atom_wm_delete, 1);
	XFlush(dpy);

	XStoreName(dpy, wnd, "bored...");
	XMapWindow(dpy, wnd);

	/* use CTRL+C to toggle urgent hint */
	memset(&sa, 0, sizeof(sa));
	sa.sa_handler = handle_sigint;
	sigaction(SIGINT, &sa, NULL);

	/* event loop */
	while (run) {
		XNextEvent(dpy, &e);

		switch (e.type) {
		case Expose:
			XClearWindow(dpy, e.xany.window);
			break;
		case ClientMessage:
			if (e.xclient.data.l[0] == (long)atom_wm_delete) {
				XUnmapWindow(dpy, wnd);
				run = 0;
			}
			break;
		}
	}

	status = 0;

	/* cleanup */
	status = EXIT_SUCCESS;
	XDestroyWindow(dpy, wnd);
outdpy:
	XCloseDisplay(dpy);
	return status;
}
